﻿using NPOI.SS.UserModel;
using Performance.DtoModels;
using Performance.Infrastructure;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Performance.Services
{
    public class PerSheetService : IAutoInjection
    {
        PerHeaderService _perHeader;
        public PerSheetService(PerHeaderService perHeader)
        {
            _perHeader = perHeader;
        }

        /// <summary>
        /// 解析excel数据
        /// </summary>
        /// <param name="sheet"></param>
        /// <returns></returns>
        public PerSheet Sheet(ISheet sheet)
        {
            PerSheet perSheet = new PerSheet();
            perSheet.SheetName = sheet.SheetName;
            perSheet.SheetType = GetSheetType(sheet.SheetName);
            perSheet.ModuleName = EnumHelper.GetDescription(perSheet.SheetType);

            if (perSheet.SheetType == SheetType.Unidentifiable)
                return null;

            var sheetRead = PerSheetDataFactory.GetDataRead(perSheet.SheetType);
            perSheet.PerHeader = _perHeader.GetPerHeader(sheet, sheetRead.Point);
            var headerReverse = _perHeader.GetPerHeaderReverse(sheet, sheetRead.Point);
            perSheet.PerData = sheetRead.ReadData(sheet, headerReverse);
            return perSheet;
        }

        /// <summary>
        /// 判断sheet适用模板类型
        /// </summary>
        /// <param name="sheetName"></param>
        /// <returns></returns>
        public SheetType GetSheetType(string sheetName)
        {
            //if (sheetName.StartsWith("医院人员名单"))
            //    return SheetType.Employee;
            //else 
            if (sheetName.StartsWith("1.0"))
                return SheetType.OtherIncome;
            //else if (sheetName.StartsWith("1.1"))
                //return SheetType.Income;
            else if (sheetName.StartsWith("2."))
                return SheetType.Expend;
            else if (sheetName.StartsWith("3."))
                return SheetType.Workload;
            //else if (sheetName.StartsWith("4.1"))
            //    return SheetType.UnifyUnit;
            return SheetType.Unidentifiable;
        }

        internal List<PerSheet> ProcessCompute(PerExcel excel)
        {
            List<PerSheet> perSheet = new List<PerSheet>();

            var economicCompute = new PerSheetDataComputeEconomic();
            var economic = economicCompute.ProcessCompute(excel);
            perSheet.Add(economic);

            var workloadCompute = new PerSheetDataComputeWorkload();
            var workload = workloadCompute.ProcessCompute(excel);
            perSheet.Add(workload);

            return perSheet;
        }
    }
}
