﻿using Performance.DtoModels;
using Performance.Infrastructure;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Performance.Services
{
    /// <summary>
    /// 工作量
    /// </summary>
    public class PerSheetDataComputeWorkload : IPerSheetDataCompute
    {
        public PerSheet ProcessCompute(PerExcel excel)
        {
            PerSheet compSheet = new PerSheet
            {
                SheetName = "工作量绩效测算表",
                ModuleName = "工作量绩效测算表",
                PerHeader = new List<PerHeader>(),
                PerData = new List<IPerData>()
            };

            //初始核算单元列头 
            compSheet.PerHeader.Add(new PerHeader { CellValue = "核算单元", Level = 0, MergeCell = 1, MergeRow = 2, PointCell = 0, PointRow = 0 });
            //起始列
            int pointCell = 1;
            //计算需从以下sheet中拿取数据
            SheetType[] types = { SheetType.Workload };
            foreach (var type in types)
            {
                //取出当前sheet类型数据
                var sheetList = excel.PerSheet.Where(t => t.SheetType == type);
                if (sheetList.Any())
                {
                    List<PerHeader> headList = new List<PerHeader>();
                    //子集列号为父级列号
                    int childPointCell = pointCell;
                    foreach (var sheet in sheetList)
                    {
                        var dataList = sheet.PerData.Select(t => (PerData)t);
                        foreach (var group in dataList.GroupBy(t => t.UnitType))
                        {
                            var typeName = $"{GetCleanSheetName(sheet.SheetName)}({ group.Key})";
                            var ds = group.GroupBy(t => t.AccountingUnit).Select(t => new PerData
                            {
                                UnitType = group.Key,
                                AccountingUnit = t.Key,
                                CellValue = t.Sum(s => s.IsFactor ? (s.CellValue * (s.FactorValue ?? 0)) : s.CellValue),
                                TypeName = typeName,
                                RowNumber = dataList.FirstOrDefault(s => s.AccountingUnit == t.Key && s.UnitType == group.Key)?.RowNumber ?? 0
                            });

                            compSheet.PerData.AddRange(ds);
                            //创建子集头部信息
                            PerHeader childHeader = new PerHeader(1, childPointCell, typeName, 1, 1, 1, null);
                            headList.Add(childHeader);
                            childPointCell++;
                        }
                    }
                    PerHeader header = new PerHeader(0, pointCell, EnumHelper.GetDescription(type), 1, 1, headList.Count, headList);
                    compSheet.PerHeader.Add(header);
                    //父级列号为当前列号+子集个数
                    pointCell = pointCell + headList.Count;
                }
            }
            return compSheet;
        }
        private string GetCleanSheetName(string sheetName)
        {
            sheetName = sheetName
               .Replace("医生组", "")
               .Replace("护理组", "")
               .Replace(".", "");

            sheetName = Regex.Replace(sheetName, @"\d", "");

            return sheetName;
        }
    }
}
