﻿using AutoMapper;
using Performance.DtoModels;
using Performance.Repository;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Performance.Services
{
    /// <summary>
    /// 提供sheet数据查询服务
    /// </summary>
    public class SheetSevice : IAutoInjection
    {
        private PerforAllotRepository _perforAllotRepository;
        private PerforImSheetRepository _perforImSheetRepository;
        private PerforImDataRepository _perforImDataRepository;
        private PerforImHeaderRepository _perforImHeaderRepository;
        private PerforImEmployeeRepository _perforImEmployeeRepository;

        public SheetSevice(PerforAllotRepository perforAllotRepository,
            PerforImSheetRepository perforImSheetRepository,
            PerforImDataRepository perforImDataRepository,
            PerforImHeaderRepository perforImHeaderRepository,
            PerforImEmployeeRepository perforImEmployeeRepository)
        {
            _perforAllotRepository = perforAllotRepository;
            _perforImSheetRepository = perforImSheetRepository;
            _perforImDataRepository = perforImDataRepository;
            _perforImHeaderRepository = perforImHeaderRepository;
            _perforImEmployeeRepository = perforImEmployeeRepository;
        }

        /// <summary>
        /// sheet 列表查询
        /// </summary>
        /// <param name="allotID"></param>
        /// <returns></returns>
        public List<SheetResponse> SheetList(int allotID)
        {
            var allot = _perforAllotRepository.GetEntity(t => t.ID == allotID);
            if (allot == null)
                throw new PerformanceException("参数allotid无效");

            var sheetList = _perforImSheetRepository.GetEntities(t => t.AllotID == allotID);
            return Mapper.Map<List<SheetResponse>>(sheetList);
        }

        /// <summary>
        /// sheet数据查询
        /// </summary>
        /// <param name="sheetID"></param>
        /// <returns></returns>
        public SheetExportResponse SheetExport(int sheetID)
        {
            var sheet = _perforImSheetRepository.GetEntity(t => t.ID == sheetID);
            if (sheet == null)
                throw new PerformanceException("参数sheetid无效");

            SheetExportResponse response = new SheetExportResponse(sheet.ID, sheet.SheetName, sheet.SheetType.Value);
            if (sheet.SheetType == (int)SheetType.Employee)
            {

                var employeeList = _perforImEmployeeRepository.GetEntities(t => t.SheetID == sheetID);

                if (employeeList != null && employeeList.Count() > 0)
                {
                    var row = new Row(0);
                    row.Data.Add(new Cell(1, "核算单元", 1, 1));
                    row.Data.Add(new Cell(2, "科室", 1, 1));
                    row.Data.Add(new Cell(3, "医生姓名", 1, 1));
                    row.Data.Add(new Cell(4, "职称", 1, 1));
                    row.Data.Add(new Cell(5, "绩效基数核算参考对象", 1, 1));
                    row.Data.Add(new Cell(6, "岗位系数", 1, 1));
                    row.Data.Add(new Cell(7, "考核得分率", 1, 1));
                    row.Data.Add(new Cell(8, "医生姓名", 1, 1));
                    row.Data.Add(new Cell(9, "出勤率", 1, 1));
                    row.Data.Add(new Cell(10, "核算单元医生数", 1, 1));
                    row.Data.Add(new Cell(11, "工作量绩效", 1, 1));
                    row.Data.Add(new Cell(12, "其他绩效", 1, 1));
                    row.Data.Add(new Cell(13, "医院奖罚", 1, 1));
                    row.Data.Add(new Cell(14, "调节系数", 1, 1));
                    row.Data.Add(new Cell(15, "发放系数", 1, 1));
                    response.Header.Add(row);
                    for (int i = 0; i < employeeList.Count(); i++)
                    {
                        var item = employeeList.ElementAt(i);
                        var rowbody = new Row(i);
                        rowbody.Data.Add(new Cell(1, item.AccountingUnit, 1, 1));
                        rowbody.Data.Add(new Cell(2, item.Department, 1, 1));
                        rowbody.Data.Add(new Cell(3, item.DoctorName, 1, 1));
                        rowbody.Data.Add(new Cell(4, item.JobTitle, 1, 1));
                        rowbody.Data.Add(new Cell(5, item.FitPeople, 1, 1));
                        rowbody.Data.Add(new Cell(6, item.PostCoefficient, 1, 1));
                        rowbody.Data.Add(new Cell(7, item.WorkTime, 1, 1));
                        rowbody.Data.Add(new Cell(8, item.ScoreAverageRate, 1, 1));
                        rowbody.Data.Add(new Cell(9, item.Attendance, 1, 1));
                        rowbody.Data.Add(new Cell(10, item.PeopleNumber, 1, 1));
                        rowbody.Data.Add(new Cell(11, item.Workload, 1, 1));
                        rowbody.Data.Add(new Cell(12, item.OthePerfor, 1, 1));
                        rowbody.Data.Add(new Cell(13, item.Punishment, 1, 1));
                        rowbody.Data.Add(new Cell(14, item.Adjust, 1, 1));
                        rowbody.Data.Add(new Cell(15, item.Grant, 1, 1));
                        response.Header.Add(rowbody);
                    }
                }
            }
            else
            {
                var headList = _perforImHeaderRepository.GetEntities(t => t.SheetID == sheetID);

                var dataList = _perforImDataRepository.GetEntities(t => t.SheetID == sheetID);

                //创建列头行
                var rowhead = new Row(0);
                response.Header.Add(rowhead);
                foreach (var header in headList.Where(t => !t.ParentID.HasValue || t.ParentID.Value == 0))
                {
                    rowhead.Data.Add(new Cell(header.PointCell.Value, header.CellValue, header.MergeRow.Value, header.MergeCell.Value));
                }
                if (headList.Any(t => t.ParentID.HasValue && t.ParentID.Value > 0))
                {
                    var rowhead2 = new Row(1);
                    response.Header.Add(rowhead2);
                    foreach (var header in headList.Where(t => t.ParentID.HasValue && t.ParentID.Value > 0))
                    {
                        rowhead2.Data.Add(new Cell(header.PointCell.Value, header.CellValue, header.MergeRow.Value, header.MergeCell.Value));
                    }
                }
                //创建数据行
                RowData rowbody = null;
                foreach (var data in dataList.OrderBy(t => t.RowNumber))
                {
                    var header = headList.Where(t => t.CellValue == data.TypeName).OrderByDescending(t => t.ParentID).FirstOrDefault();
                    if (rowbody == null || rowbody.Rownumber != data.RowNumber)
                    {
                        rowbody = new RowData(data.RowNumber.Value, data.AccountingUnit);
                        response.Row.Add(rowbody);
                    }
                    rowbody.Data.Add(new Cell(header.PointCell.Value, data.CellValue, 1, 1));
                }
            }

            return response;
        }
    }
}
